@@ -14,6 +14,7 @@ import uuid
 from contextlib import contextmanager
 from datetime import datetime, timezone
 import time
+from distutils.util import strtobool
 
 import h5py
 import numpy as np
@@ -56,21 +57,21 @@ chunk_sizes = {
 
 
 
-@numba.njit
-def _safe_map(data_field, map_field, map_filter=None):
-    result = np.zeros_like(map_field, dtype=data_field.dtype)
-    empty_val = result[0]
-    if map_filter is not None:
-        for i in range(len(map_field)):
-            if map_filter[i]:
-                result[i] = data_field[map_field[i]]
-            else:
-                result[i] = empty_val
-    else:
-        for i in range(len(map_field)):
-            result[i] = data_field[map_field[i]]
-
-    return result
+# @numba.njit
+# def _safe_map(data_field, map_field, map_filter=None):
+#     result = np.zeros_like(map_field, dtype=data_field.dtype)
+#     empty_val = result[0]
+#     if map_filter is not None:
+#         for i in range(len(map_field)):
+#             if map_filter[i]:
+#                 result[i] = data_field[map_field[i]]
+#             else:
+#                 result[i] = empty_val
+#     else:
+#         for i in range(len(map_field)):
+#             result[i] = data_field[map_field[i]]
+#
+#     return result
 
 
 # def str_to_float(value):
@@ -102,6 +103,14 @@ def try_str_to_int(value, invalid=0):
         return False, invalid
 
 
+def try_str_to_bool(value, invalid=0):
+    try:
+        v = bool(strtobool(value))
+        return True, v
+    except ValueError:
+        return False, invalid
+
+
 def try_str_to_float(value, invalid=0):
     try:
         v = float(value)
@@ -397,9 +406,7 @@ def _apply_spans_min(spans, src_array, dest_array):
 
 @njit
 def _apply_spans_concat(spans, src_index, src_values, dest_index, dest_values,
-                        max_index_i, max_value_i, s_start):
-    separator = np.frombuffer(b',', dtype=np.uint8)[0]
-    delimiter = np.frombuffer(b'"', dtype=np.uint8)[0]
+                        max_index_i, max_value_i, s_start, separator, delimiter):
     if s_start == 0:
         index_i = np.uint32(1)
         index_v = np.int64(0)
@@ -418,7 +425,7 @@ def _apply_spans_concat(spans, src_index, src_values, dest_index, dest_values,
         dest_index[index_i] = next_src_i
         index_i += 1
 
-        if next_src_i - cur_src_i > 1:
+        if next_src_i - cur_src_i > 0:
             if next - cur == 1:
                 # only one entry to be copied, so commas not required
                 next_index_v = next_src_i - cur_src_i + np.int64(index_v)
@@ -726,7 +733,7 @@ class DataStore:
             val._check_is_appropriate_writer_if_set(self, 'writer', reader, writer)
 
             src_indices = reader.field['index'][:]
-            src_values = reader.field.get('values', np.zeros(0, dtype='S1'))[:]
+            src_values = reader.field.get('values', np.zeros(0, dtype=np.uint8))[:]
 
             indices, values = _apply_indices_to_index_values(indices_to_apply,
                                                              src_indices, src_values)
@@ -969,11 +976,20 @@ class DataStore:
 
         max_index_i = reader.chunksize
         max_value_i = reader.chunksize * 8
+
+        if src_values.dtype == 'S1':
+            separator = b','
+            delimiter = b'"'
+        elif src_values.dtype == np.uint8:
+            separator = np.frombuffer(b',', dtype='S1')[0][0]
+            delimiter = np.frombuffer(b'"', dtype='S1')[0][0]
+
         s = 0
         while s < len(spans) - 1:
             s, index_i, index_v = _apply_spans_concat(spans, src_index, src_values,
                                                       dest_index, dest_values,
-                                                      max_index_i, max_value_i, s)
+                                                      max_index_i, max_value_i, s,
+                                                      separator, delimiter)
 
             if index_i > 0 or index_v > 0:
                 writer.write_raw(dest_index[:index_i], dest_values[:index_v])
